package com.chinacreator.datasync.dao;

import java.util.List;

import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import com.chinacreator.datasync.entity.CpAccountBean;
import com.chinacreator.datasync.entity.CpSyncJobBean;
import com.chinacreator.datasync.entity.OrderBean;
import com.chinacreator.datasync.entity.SpBean;
import com.chinacreator.datasync.entity.SpCpBean;
import com.chinacreator.datasync.fakeid.CpFakeidBean;
import com.chinacreator.datasync.fakeid.CpFakeidLog;

@Mapper
public interface QueryDbDao {
	
	/**
	 * 查询线上待同步数据
	 * 
	 * @param size
	 * @return
	 */
	public List<OrderBean> pageQueryOnline(@Param(value = "pagesize")int size);
	
	/**
	 * 查询线下待同步数据
	 * 
	 * @param size
	 * @return
	 */
	public List<OrderBean> pageQueryOffline(@Param(value = "pagesize")int size);
	
	/**
	 * 查询线下待同步数据
	 * 
	 * @param size
	 * @return
	 */
	public List<OrderBean> pageQueryCdn(@Param(value = "pagesize")int size);
	
	/**
	 * 产品信息
	 * @param spid
	 * @return
	 */
	public SpBean queryBySpid(@Param(value = "spid")String spid);
	
	/**
	 * 查询子产品信息
	 * 
	 * @param spid
	 * @return
	 */
	public List<SpCpBean> querySpCpBeans(@Param(value = "spid")String spid);
	
	/**
	 * 查询cp账号信息
	 * 
	 * @param cpid
	 * @return
	 */
	public CpAccountBean queryCpAccount(@Param(value = "cpid")String cpid);
	
	/**
	 * 查询cp同步job
	 * @param cpid
	 * @param dealtype
	 * @return
	 */
	public CpSyncJobBean queryCpSyncJob(@Param(value = "cpid")String cpid,@Param(value = "dealtype")String dealtype);
	
	/**
	 * 更新当前同步状态
	 * 
	 * @return
	 */
	public int updateOrderSyncStatus(@Param(value = "id")String id,@Param(value = "syncstatus")String syncstatus);
	public int updateOrderSyncStatusP(@Param(value = "id")String id,@Param(value = "syncstatus")String syncstatus,@Param(value = "status")String status);
	
	/**
	 * 更新当前同步状态
	 * 
	 * @return
	 */
	public int updateOrderSyncStatusCDN(@Param(value = "id")String id);
	
	
	/**
	 * 查询电商伪码
	 * @param userid
	 * @param cpid
	 * @return
	 */
	public CpFakeidBean findFakeidBean(@Param(value = "userid")String userid, @Param(value = "cpid")String cpid);
	
	
	/**
	 * 保存电商伪码查询日志
	 * 
	 * @param log
	 * @return
	 */
	public int insertFakeidLog(CpFakeidLog log);
	
	
	/**
	 * 保存电商伪码信息
	 * 
	 * @param bean
	 * @return
	 */
	public int insertFakeidBean(CpFakeidBean bean);
	
	/**
	 * 更新订购关系表的 fakeids字段
	 * @param id
	 * @param fakeids
	 * @return
	 */
	public int updateFakeids(@Param(value = "id")String id,@Param(value = "fakeids")String fakeids);
	
	
	/**
	 * 一般产品  - 保存标记数据
	 * 
	 * @param syncid
	 * @param id
	 * @param userid
	 * @return
	 */
	public int insertSyncFlagBean(@Param(value = "syncid")String syncid,
			@Param(value = "orderid")String id,
			@Param(value = "userid")String userid);
	
	/**
	 *  复合产品  - 保存标记数据
	 *  
	 * @param syncid
	 * @param id
	 * @param userid
	 * @return
	 */
	public int insertSyncFlagBeans(@Param(value = "syncid")String syncid,
			@Param(value = "orderid")String id,
			@Param(value = "userid")String userid,
			@Param(value = "spid")String spid);
	
	/**
	 * 一般产品  - 保存标记数据
	 * 
	 * @param syncid
	 * @param id
	 * @param userid
	 * @return
	 */
	public int insertSyncFlagBeanCdn(@Param(value = "syncid")String syncid,
			@Param(value = "orderid")String id,
			@Param(value = "userid")String userid);
	
	/**
	 *  复合产品  - 保存标记数据
	 *  
	 * @param syncid
	 * @param id
	 * @param userid
	 * @return
	 */
	public int insertSyncFlagBeansCdn(@Param(value = "syncid")String syncid,
			@Param(value = "orderid")String id,
			@Param(value = "userid")String userid,
			@Param(value = "spid")String spid);
	
	/**
	 * 入队列失败，删除同步标记数据
	 * 
	 * @param syncid
	 * @param id
	 * @param userid
	 * @return
	 */
	public int deleteSyncFlagBeans(@Param(value = "syncid")String syncid,
			@Param(value = "orderid")String id,
			@Param(value = "userid")String userid);

	/**
	 * 获取当前的fakeids
	 *
	 * @param id
	 * @return
	 */
	public String getNowfakeids(@Param(value = "id")String id);

	/**
	 * 查询待同步的伪码数据
	 *
	 * @param size
	 * @return
	 */
	public List<OrderBean> pageQueryFakeids(@Param(value = "pagesize")int size);

	/**
	 * 更新伪码数据
	 *
	 * @param size
	 * @return
	 */
	public int updateFakeidsSyncStatus(@Param(value = "id")String id,@Param(value = "syncstatus")String syncstatus,@Param(value = "syncinfo")String syncinfo);
	public int updateFakeidsSyncStatusP(@Param(value = "id")String id,@Param(value = "syncstatus")String syncstatus,@Param(value = "status")String status);

	/**
	 * 查询待同步的伪码数据和订购关系表比对是否正确
	 *
	 * @return
	 */
	public List<OrderBean> judgeSyncFakeids(@Param(value = "userid")String userid, @Param(value = "spid")String spid);

	/**
	 * 查询待同步的伪码数据在同步表内是否已经同步过相关产品
	 *
	 * @return
	 */
	public int judgeFakeidsHasSync(@Param(value = "userid")String userid, @Param(value = "spid")String spid,@Param(value = "productid")String productid);

	/**
	 * 查询复合产品需要同步的子产品id （productid）
	 *
	 * @return
	 */
	public String findSyncProductid(@Param(value = "spid")String spid);

	/**
	 * 更新同步表数据（根据最新订购关系）
	 *
	 * @return
	 */
	public int updateSyncRec(OrderBean orderBean);

	/**
	 * 保存bilibl电商伪码信息
	 *
	 * @param bean
	 * @return
	 */
	public int insertBiliblFakeidBean(CpFakeidBean bean);
}
