package com.chinacreator.datasync.handler;

import com.chinacreator.datasync.common.ConfigUtil;
import com.chinacreator.datasync.dao.QueryDbDao;
import com.chinacreator.datasync.entity.OrderBean;
import com.chinacreator.datasync.entity.SpBean;
import org.slf4j.Logger;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.CountDownLatch;


/**
 * 
 * 伪码同步数据处理
 * 
 * @author zhengrong.yan
 *
 */
public class SyncFakeids implements Runnable {

	private Logger logger ;

	QueryDbDao dao ;

	private OrderBean orderBean;
	private int totalSize;
	private CountDownLatch threadSignal;

	public static Map<String,Object> map = new HashMap<String, Object>();

	private ISyncFakeidsHandler syncFakeidsCommonHandler = new SyncFakeidsCommonHandler();
	private ISyncFakeidsHandler syncFakeidsMultiHandler = new SyncFakeidsMultiHandler();

	public SyncFakeids(OrderBean orderBean, int totalSize, CountDownLatch threadSignal, Logger logger, QueryDbDao dao) {
		this.logger = logger;
		
		this.dao = dao;
		
		this.orderBean = orderBean;
		this.totalSize = totalSize;
		this.threadSignal = threadSignal;
	}
	
	public void run() {
		threadSignal.countDown();
		long curSignal = threadSignal.getCount();
		
		boolean result = false;
		long start = System.currentTimeMillis();
		SpBean spBean = ConfigUtil.queryBySpid(this.orderBean.getSpid(), dao);
		if(spBean == null ){
			logger.info("暂无该产品: cpid ==> " + orderBean.getCpid() + ", spid ==> " + orderBean.getSpid());
			return ;
		}
		
		try {
			
			//同步唯一标识
			String syncid = System.currentTimeMillis() + String.valueOf((int)((Math.random()*9+1)*10000000));
			orderBean.setSyncid(syncid);
			
			try {
				//复合产品
				if("2".equals(spBean.getType()) || "4".equals(spBean.getType())) {
					logger.info("是复合产品"+orderBean.getUserid()+", "+orderBean.getSpid()+", "+spBean.getType());
					result = syncFakeidsMultiHandler.doSyncFakeids(orderBean, dao, logger);
				} else {
					logger.info("非复合产品"+orderBean.getUserid()+", "+orderBean.getSpid()+", "+spBean.getType());
					result = syncFakeidsCommonHandler.doSyncFakeids(orderBean, dao, logger);
				}
			} catch (Exception e) {
				logger.error("处理异常, "+e.getMessage()+", "+orderBean.getUserid()+", "+orderBean.getSpid(), e);
			}
			
			//处理结果
			if(result) {
				logger.info("推送到kafka成功，更新伪码同步表"+orderBean.getUserid()+", "+orderBean.getSpid());
				dao.updateFakeidsSyncStatusP(orderBean.getId(), "P",orderBean.getStatus());
			}else {
				logger.info("推送kafka失败，更新伪码同步表为失败，"+orderBean.getUserid()+", "+orderBean.getSpid());
				dao.updateFakeidsSyncStatus(orderBean.getId(), "1","推送kafka失败");
			}
		} catch (Exception e) {
			logger.error("处理异常2, "+e.getMessage()+", "+orderBean.getUserid()+", "+orderBean.getSpid(), e);
		} finally {
			logger.info("线程名---->"+Thread.currentThread().getName());
			logger.info("[" + (totalSize -  curSignal) + "/" + totalSize + "]同步数据入队列处理" + (result ? "成功" : "失败") + "，花费时间：" + (System.currentTimeMillis() - start) + " ms ,>>>>>>"+orderBean.getUserid());
		}
	}
	
	


}
