package com.chinacreator.datasync.handler;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.chinacreator.datasync.common.CommonUtil;
import com.chinacreator.datasync.common.ConfigUtil;
import com.chinacreator.datasync.dao.QueryDbDao;
import com.chinacreator.datasync.entity.CpSyncJobBean;
import com.chinacreator.datasync.entity.OrderBean;
import com.chinacreator.datasync.entity.OrderSyncKafkaBean;
import com.chinacreator.datasync.exception.SyncException;
import com.chinacreator.datasync.fakeid.CpFakeidUtil;
import com.chinacreator.datasync.handler.process.ISyncOrderProcessor;
import com.chinacreator.datasync.handler.process.SyncOrderProcessor;
import com.chinacreator.datasync.util.SpringContextUtils;
import org.slf4j.Logger;
import org.springframework.kafka.core.KafkaTemplate;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;


/**
 * @author zhengrong.yan
 * @Description: 单产品推入kafka同步
 * @date 20211105
 **/
public class SyncFakeidsCommonHandler implements ISyncFakeidsHandler {

    public static String fakeCpids = ",bilibl,";

    public static String cp_fakeids_topic = "cp_fakeids_topic";

    @Override
    public boolean doSyncFakeids(OrderBean bean, QueryDbDao dao, Logger logger) throws Exception {
        int ordertype = 1;
        return synFakeids(bean, ordertype, bean.getCpid(), bean.getSpid(), dao, logger);
    }


    public boolean synFakeids(OrderBean orderBean, int ordertype, String cpid, String spid, QueryDbDao dao, Logger logger) throws Exception {

        /**
         * 电商伪码处理
         */
        if (fakeCpids.contains("," + orderBean.getCpid() + ",")) {
            String fakeid = null;
            JSONObject fakeJson = null;
            logger.info("fakeids ===>" + orderBean.getFakeids());
            if (orderBean.getFakeids() != null && !"".equals(orderBean.getFakeids().trim())) {
                try {
                    fakeJson = JSONObject.parseObject(orderBean.getFakeids().trim());
                    if (fakeJson.getString(orderBean.getCpid()) != null && !"".equals(fakeJson.getString(orderBean.getCpid()))) {
                        fakeid = fakeJson.getString(orderBean.getCpid());
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }

            if (fakeid == null || "".equals(fakeid.trim())) {
                String fake = CpFakeidUtil.fakeidQuery(orderBean.getUserid(), orderBean.getCpid(), orderBean.getSpid(), dao);
                if (fake != null && !"".equals(fake.trim()) && !"CPNULL".equals(fake.trim())) {
                    fakeid = fake;
                    logger.info("-----orderbean.cpid " + cpid + "  fakeid--->" + fakeid);
                    try {
                        if (fakeJson == null) {
                            fakeJson = new JSONObject();
                        }
                        if ("bilibl".equals(orderBean.getCpid())) {
                            logger.info("----bilibili---");
                            String now_month = (new SimpleDateFormat("YYYYMM")).format(new Date());
                            fakeJson.put(orderBean.getCpid() + "_" + now_month, fakeid);//将本月的put进去
                            //查看是否有小于前两个月的,有则删除
                            String last2month = getLast12Months(2);
                            if (fakeJson.get("bilibl_" + last2month) != null) {
                                fakeJson.remove("bilibl_" + last2month);
                            }
                        } else {
                            fakeJson.put(orderBean.getCpid(), fakeid);
                        }
                        logger.info("------fakejson---" + fakeJson.toString());
                        dao.updateFakeids(orderBean.getOrderid(), fakeJson.toString());
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                } else {
                    logger.info("未获取到fakeid，流程中止， " + orderBean.getUserid() + ", " + orderBean.getSpid());
                    return false;
                }
            }

            orderBean.setFakeid(fakeid);
        }

        /**
         * 数据入kafka
         */
        long start = System.currentTimeMillis();
        ISyncOrderProcessor processor = SyncOrderProcessor.getInstance().getSyncOrderProcessor(orderBean.getCpid());
        boolean result = false;
        String errorinfo = null;
        try {
            logger.info("开始推送到kafka，" + orderBean.getUserid() + ", " + orderBean.getSpid());
            result = kafkaProduce(orderBean, processor, ordertype, cpid, spid, dao, logger);
        } catch (Exception e) {
            try {
                errorinfo = e.getClass().getName() + ":" + e.getMessage().toString();
                if (errorinfo.length() > 500) {
                    errorinfo = errorinfo.substring(0, 500);
                }
            } catch (Exception e2) {
            }
        }

        logger.info("同步数据压入队列" + (result ? "成功" : "失败") + ",times=" + (System.currentTimeMillis() - start) + (errorinfo == null ? "" : ("," + errorinfo)) + orderBean.getUserid() + ", " + orderBean.getSpid());
        return result;
    }


    public boolean kafkaProduce(OrderBean orderBean, ISyncOrderProcessor processor, int ordertype, String fromCpid, String fromSpid, QueryDbDao dao, Logger logger) throws Exception {
        boolean ret = false;
        OrderSyncKafkaBean syncOrderInfo = null;
        try {
            syncOrderInfo = processor.toCPSyncOrderInfo(orderBean, ordertype, dao, logger);
            syncOrderInfo.setFromCpid(fromCpid);
            syncOrderInfo.setFromSpid(fromSpid);
        } catch (Exception e) {
            logger.error("[数据异常]生成的同步数据不符合规范, orderid=" + orderBean.getId() + ",userid=" + orderBean.getUserid() + ",cpid=" + orderBean.getCpid() + ",spid=" + orderBean.getSpid(), e);
            throw new SyncException("1003", "[数据异常]生成的同步数据不符合规范, orderid=" + orderBean.getId() + ",userid=" + orderBean.getUserid() + ",cpid=" + orderBean.getCpid() + ",spid=" + orderBean.getSpid());
        }
        CpSyncJobBean orderJobInfo = null;
        orderJobInfo = ConfigUtil.queryFakeidsSyncJob(orderBean.getCpid(), dao);

        if (orderJobInfo != null) {
            try {
                KafkaTemplate<String, String> producter = (KafkaTemplate<String, String>) SpringContextUtils.getBean("KafkaProducter");
                //cp数据 入kafka
                syncOrderInfo.setSynctype("cp");
                syncOrderInfo.setSyncid(syncOrderInfo.getId());
                producter.send(cp_fakeids_topic, ConfigUtil.getPartition(orderBean.getUserid(), 3), null, JSON.toJSONString(syncOrderInfo));
                logger.info("cp fakeidsdata " + JSON.toJSONString(syncOrderInfo));
                ret = true;
            } catch (Exception e) {
                logger.error("Kafka生产者生产异常：" + e.getClass().getName() + "|" + e.getMessage() + ", " + orderBean.getUserid() + ", " + orderBean.getSpid(), e);
                throw new SyncException("1002", "Kafka生产者生产异常：" + e.getClass().getName() + "|" + e.getMessage());
            }
        } else {
            throw new SyncException("1001", "CP同步[cpid=" + orderBean.getCpid() + ",type=fakeidssync" + "]对应job配置不存在");
        }
        return ret;
    }

    /*获得前两个月的月份*/
    public static String getLast12Months(int i) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMM");
        Calendar c = Calendar.getInstance();
        c.setTime(new Date());
        c.add(Calendar.MONTH, -i);
        Date m = c.getTime();
        return sdf.format(m);
    }
}
