package com.chinacreator.datasync.handler;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.CountDownLatch;

import org.slf4j.Logger;

import com.chinacreator.datasync.common.ConfigUtil;
import com.chinacreator.datasync.dao.QueryDbDao;
import com.chinacreator.datasync.entity.OrderBean;
import com.chinacreator.datasync.entity.SpBean;


/**
 * 
 * 同步数据处理
 * 
 * @author maoya
 *
 */
public class SyncOrder implements Runnable {
	
	private Logger logger ;
	
	QueryDbDao dao ;
	
	private OrderBean orderBean;
	private int totalSize;
	private CountDownLatch threadSignal;
	
	public static Map<String,Object> map = new HashMap<String, Object>();
	
	private ISyncOrderHandler syncOrderCommonHandler = new SyncOrderCommonHandler();
	private ISyncOrderHandler syncOrderMultiHandler = new SyncOrderMultiHandler();

	public SyncOrder(OrderBean orderBean, int totalSize, CountDownLatch threadSignal,Logger logger, QueryDbDao dao) {
		this.logger = logger;
		
		this.dao = dao;
		
		this.orderBean = orderBean;
		this.totalSize = totalSize;
		this.threadSignal = threadSignal;
	}
	
	public void run() {
		threadSignal.countDown();
		long curSignal = threadSignal.getCount();
		
		boolean result = false;
		long start = System.currentTimeMillis();
		SpBean spBean = ConfigUtil.queryBySpid(this.orderBean.getSpid(), dao);
		if(spBean == null ){
			logger.info("暂无该产品: cpid ==> " + orderBean.getCpid() + ", spid ==> " + orderBean.getSpid());
			return ;
		}
		
		//订购状态为空，不处理，否则更新不了订购关系表的同步状态为P，造成同一数据反复被推送到KAFKA
		if(orderBean.getStatus() == null || "".equals(orderBean.getStatus())){
			logger.info("同步产品无订购状态，不处理, id =>"+orderBean.getId()+", userid =>"+orderBean.getUserid()+", status =>" + orderBean.getStatus());
			try {
				//直接设置为同步失败
				dao.updateOrderSyncStatus(orderBean.getId(), "1");
			} catch (Exception e) {
				e.printStackTrace();
				logger.error("直接设置为同步失败出现异常，id=>"+orderBean.getId()+", userid =>"+orderBean.getUserid()+", 异常信息=>"+e.getMessage());
			}
			return ;
		}
		
		try {
			if("tencent".equals(orderBean.getCpid()) && "101".equals(orderBean.getSpid())){
				orderBean.setSpid("84");
			}
			
			//同步唯一标识
			String syncid = System.currentTimeMillis() + String.valueOf((int)((Math.random()*9+1)*10000000));
			orderBean.setSyncid(syncid);
			
			try {
				//复合产品
				if("2".equals(spBean.getType()) || "4".equals(spBean.getType())) {
					logger.info("是复合产品"+orderBean.getUserid()+", "+orderBean.getSpid()+", "+spBean.getType());
					result = syncOrderMultiHandler.doSyncOrder(orderBean, dao, logger);
				} else {
					logger.info("非复合产品"+orderBean.getUserid()+", "+orderBean.getSpid()+", "+spBean.getType());
					result = syncOrderCommonHandler.doSyncOrder(orderBean, dao, logger);
				}
			} catch (Exception e) {
				logger.error("处理异常, "+e.getMessage()+", "+orderBean.getUserid()+", "+orderBean.getSpid(), e);
			}
			
			//处理结果
			if(result) {
				logger.info("推送到kafka成功，更新订购关系表"+orderBean.getUserid()+", "+orderBean.getSpid());
				dao.updateOrderSyncStatusP(orderBean.getId(), "P",orderBean.getStatus());
//				if("0".equals(orderBean.getCdnflag())) {
//					dao.updateOrderSyncStatusCDN(orderBean.getId());
//				}
			}else {
				logger.info("推送kafka失败，更新订购关系表为失败，删除同步待处理表对应数据，"+orderBean.getUserid()+", "+orderBean.getSpid());
				dao.updateOrderSyncStatus(orderBean.getId(), "1");
				dao.deleteSyncFlagBeans(syncid, orderBean.getId(), orderBean.getUserid());
			}
		} catch (Exception e) {
			logger.error("处理异常2, "+e.getMessage()+", "+orderBean.getUserid()+", "+orderBean.getSpid(), e);
		} finally {
			logger.info("[" + (totalSize -  curSignal) + "/" + totalSize + "]同步数据入队列处理" + (result ? "成功" : "失败") + "，花费时间：" + (System.currentTimeMillis() - start) + " ms ,>>>>>>"+orderBean.getUserid());
		}
	}
	
	


}
