package com.chinacreator.datasync.services;

import java.util.List;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import com.chinacreator.datasync.common.CommonUtil;
import com.chinacreator.datasync.dao.QueryDbDao;
import com.chinacreator.datasync.entity.OrderBean;
import com.chinacreator.datasync.handler.SyncOrder;

@Component
public class SyncCdnService {
	
	public Logger logger = LoggerFactory.getLogger("cdnSync");
	
	@Value("${config.query.cdn.pagesize}")
	public int size;
	
	@Autowired
    private QueryDbDao dao;
	
	public void sync(){
		logger.info("SyncCdnJob 开始 ............");
		
		//查询需要同步的数据 
		List<OrderBean> list = dao.pageQueryCdn(size);
		if (!CommonUtil.isNotBlank(list)) {
			logger.info("SyncCdnJob 无需要同步的数据 ............");
			CommonUtil.sleep(20000,logger);
			return;
		}
		
		//多程处理需要同步的数据
		logger.info("SyncCdnJob  datasize ==> " + list.size());
		CountDownLatch threadSignal = new CountDownLatch(list.size());
		ExecutorService executorService = new ThreadPoolExecutor(5, 20, 10L, TimeUnit.MINUTES, new LinkedBlockingQueue<Runnable>(), new ThreadPoolExecutor.CallerRunsPolicy());
		for (OrderBean orderBean : list) {
			orderBean.setCdnflag("0");
			SyncOrder syncOrder = new SyncOrder(orderBean, list.size(), threadSignal, logger, dao);
			executorService.execute(syncOrder);
		}
		
		//等待多线程处理完
		executorService.shutdown();
		try {
			executorService.awaitTermination(20L, TimeUnit.MINUTES);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		
	}
	
	
	
}
