package com.chinacreator.datasync.services;

import com.chinacreator.datasync.common.CommonUtil;
import com.chinacreator.datasync.common.ConfigUtil;
import com.chinacreator.datasync.dao.QueryDbDao;
import com.chinacreator.datasync.entity.OrderBean;
import com.chinacreator.datasync.entity.SpBean;
import com.chinacreator.datasync.handler.SyncFakeids;
import com.chinacreator.datasync.handler.SyncOrder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.concurrent.*;

/**
 * @author zhengrong.yan
 * @date 2021/11/5 9:49
 */

@Component
public class SyncFakeidsService {

    public Logger logger = LoggerFactory.getLogger("fakeidsSync");

    @Value("${config.query.fakeids.pagesize}")
    public int size;

    @Value("${config.query.fakeids.corepoolsize}")
    public int corepoolsize;

    @Value("${config.query.fakeids.maximumpoolsize}")
    public int maximumpoolsize;

    @Autowired
    private QueryDbDao dao;

    public void sync(){
        logger.info("SyncFakeidsJob 开始 ............");

        //查询需要同步的数据
        List<OrderBean> list = dao.pageQueryFakeids(size);
        if (!CommonUtil.isNotBlank(list)) {
            logger.info("SyncFakeidsJob 无需要同步的数据 ............");
            CommonUtil.sleep(20000,logger);
            return;
        }

        //多程处理需要同步的数据
        logger.info("SyncFakeidsJob  datasize ==> " + list.size());
        try {
            CountDownLatch threadSignal = new CountDownLatch(list.size());
            ExecutorService executorService = new ThreadPoolExecutor(corepoolsize, maximumpoolsize, 10L, TimeUnit.MINUTES, new LinkedBlockingQueue<Runnable>(), new ThreadPoolExecutor.CallerRunsPolicy());
            for (OrderBean orderBean : list) {
                //检查是否是复合产品
                SpBean spBean = ConfigUtil.queryBySpid(orderBean.getSpid(), dao);
                //判断和当前订购关系的情况，如果需要处理则进行同步，不需要处理则过滤掉
                List<OrderBean> relationsbeans = dao.judgeSyncFakeids(orderBean.getUserid(), orderBean.getSpid());
                if(relationsbeans!=null && relationsbeans.size()>0) {
                    OrderBean relationsbean = relationsbeans.get(0);
                    if (relationsbean != null && orderBean.getUserid().equals(relationsbean.getUserid())) {
                        //是复合产品
                        if (spBean != null && ("2".equals(spBean.getType()) || "4".equals(spBean.getType()))) {
                            logger.info("是复合产品，开始检查是否已有同步过的子产品或其他复合产品");
                            String productid = dao.findSyncProductid(orderBean.getSpid());
                            int count = dao.judgeFakeidsHasSync(orderBean.getUserid(), orderBean.getSpid(), productid);
                            if (count > 0) {
                                //表明已经无需同步，其他产品已同步过
                                logger.info(orderBean.getUserid() + "===>哔哩其他子产品或复合产品已同步过伪码");
                                dao.updateFakeidsSyncStatus(orderBean.getId(), "2", "该手机号哔哩其他子产品或复合产品已同步过伪码");
                                continue;
                            }
                        }
                        //更新同步表的数据，然后开始同步
                        relationsbean.setId(orderBean.getId());
                        logger.info(relationsbean.toString());
                        if ("".equals(relationsbean.getEndtime()) && "".equals(relationsbean.getCanceltime())) {
                            relationsbean.setCanceltime(null);
                            relationsbean.setEndtime(null);
                        }
                        dao.updateSyncRec(relationsbean);
                        logger.info("---更新表结束，开始同步伪码---");
                        SyncFakeids syncFakeids = new SyncFakeids(orderBean, list.size(), threadSignal, logger, dao);
                        executorService.execute(syncFakeids);
                    }
                }else {
                    //表明已经无需同步，订购关系表状态已变
                    logger.info(orderBean.getUserid() + "===>无需同步，该产品订购关系已更改");
                    dao.updateFakeidsSyncStatus(orderBean.getId(), "2", "无需同步，该产品订购关系已更改");
                }
            }
            //等待多线程处理完
            executorService.shutdown();
            try {
                executorService.awaitTermination(20L, TimeUnit.MINUTES);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }catch (Exception e){
            e.printStackTrace();
        }
    }
}
