package com.chinacreator.datasync.services;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.stereotype.Component;

import com.chinacreator.datasync.common.CommonUtil;
import com.chinacreator.datasync.dao.QueryDbDao;
import com.chinacreator.datasync.entity.OrderBean;
import com.chinacreator.datasync.entity.SpBean;
import com.chinacreator.datasync.handler.SyncOrder;

@Component
public class SyncOfflineService {
	
public Logger logger = LoggerFactory.getLogger("offlineSync");
	
	@Value("${config.query.offline.pagesize}")
	public int size;
	
	@Autowired
    private QueryDbDao dao;
	
	@Autowired
	private ReplaceChannelService service;
	
	public static Map<String,Object> map = new HashMap<String, Object>();
	
	public void sync(){
		//查询需要同步的数据 
		List<OrderBean> list = dao.pageQueryOffline(size);
		if (!CommonUtil.isNotBlank(list)) {
			logger.info("SyncOfflineJob 无需要同步的数据 ............");
			CommonUtil.sleep(100,logger);
			return;
		}
		
		//多程处理需要同步的数据
		logger.info("SyncOfflineJob  datasize ==> " + list.size());
		CountDownLatch threadSignal = new CountDownLatch(list.size());
		ExecutorService executorService = new ThreadPoolExecutor(15, 25, 10L, TimeUnit.MINUTES, new LinkedBlockingQueue<Runnable>(), new ThreadPoolExecutor.CallerRunsPolicy());
		for (OrderBean orderBean : list) {
			//替换要同步的渠道标识，柏源渠道用
			service.replaceChannel(orderBean);
			SyncOrder syncOrder = new SyncOrder(orderBean, list.size(), threadSignal, logger, dao);
			executorService.execute(syncOrder);
		}
		
		//等待多线程处理完
		executorService.shutdown();
		try {
			executorService.awaitTermination(20L, TimeUnit.MINUTES);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		
	}
	
	
	
}
