package com.chinacreator.datasync.services;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import com.chinacreator.datasync.common.CommonUtil;
import com.chinacreator.datasync.dao.QueryDbDao;
import com.chinacreator.datasync.entity.OrderBean;
import com.chinacreator.datasync.handler.SyncOrder;

@Component
public class SyncOnlineService {
	
	public Logger logger = LoggerFactory.getLogger("onlineSync");
	
	@Value("${config.query.online.pagesize}")
	public int size;
	
	@Autowired
    private QueryDbDao dao;
	
	@Autowired
	private ReplaceChannelService service;
	
	private SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");
	
	public void sync(){
		//查询需要同步的数据 
		List<OrderBean> list = dao.pageQueryOnline(size);
		if (!CommonUtil.isNotBlank(list)) {
			logger.info("SyncOnlineJob 无需要同步的数据 ............");
			CommonUtil.sleep(100,logger);
			return;
		}
		
		//多程处理需要同步的数据
		logger.info("SyncOnlineJob  datasize ==> " + list.size()+"， "+format.format(new Date()));
		printdata(list);
		CountDownLatch threadSignal = new CountDownLatch(list.size());
		ExecutorService executorService = new ThreadPoolExecutor(15, 25, 10L, TimeUnit.MINUTES, new LinkedBlockingQueue<Runnable>(), new ThreadPoolExecutor.CallerRunsPolicy());
		for (OrderBean orderBean : list) {
			//替换要同步的渠道标识，柏源渠道用
			service.replaceChannel(orderBean);
			SyncOrder syncOrder = new SyncOrder(orderBean, list.size(), threadSignal, logger, dao);
			executorService.execute(syncOrder);
		}
		
		//等待多线程处理完
		executorService.shutdown();
		try {
			executorService.awaitTermination(20L, TimeUnit.MINUTES);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		logger.info("SyncOnlineJob  处理完成 " +"， "+format.format(new Date()));
	}
	
	private void printdata(List<OrderBean> list){
		if(list != null && list.size()>0){
			for(OrderBean tmp : list){
				logger.info(tmp.getId()+"，"+tmp.getUserid());
			}
		}
	}
	
}
